from pydantic import BaseModel
from bson.objectid import ObjectId
from fastapi.security import OAuth2PasswordBearer
from fastapi import HTTPException, status
from fastapi.param_functions import Depends
from datetime import timedelta, datetime
from fastapi.security.oauth2 import SecurityScopes
from config import ENVIRONMENT, SECRET_KEY, ALGORITHM, BASEURL

import jwt


oauth2_scheme = OAuth2PasswordBearer(
    tokenUrl=BASEURL+"crmv2/main_auth/auth/login"
)

CREDENTIALS_EXCEPTION = HTTPException(
    status_code=status.HTTP_401_UNAUTHORIZED,
    detail="Sesi telah berakhir, silahkan login kembali!",
    headers={"WWW-Authenticate": "Bearer"},
)

ROLE_EXCPETION = HTTPException(
    status_code=status.HTTP_401_UNAUTHORIZED,
    detail="Role tidak memiliki akses",
    headers={"WWW-Authenticate": "Bearer"},
)

ACCESS_TOKEN_EXPIRE_MINUTES = 15


# =============================class=============================
class ObjectIdStr(str):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v):
        if type(v) == str:
            v = ObjectId(v)
        if not isinstance(v, ObjectId):
            raise ValueError("Not a valid ObjectId")
        return str(v)


class JwtToken(BaseModel):
    name: str = None
    username: str = None
    email: str = None
    schoolId: ObjectIdStr = None
    bankCentralId: ObjectIdStr = None
    bankAreaId: ObjectIdStr = None
    bankBranchId: ObjectIdStr = None
    userId : ObjectIdStr = None
    # teamAreaId: ObjectIdStr = None
    role : str = None
    exp : int = 1000

# =============================controller=============================
async def mytoken(token: str = Depends(oauth2_scheme)):
    return token

async def create_access_token(data: JwtToken, expires_delta: int, ALGORITHM='HS256'):
    if expires_delta:
        expire = datetime.utcnow() + timedelta(minutes=expires_delta)
    else:
        expire = datetime.utcnow() + timedelta(minutes=1)
    data.exp = expire
    encoded_jwt = jwt.encode(data.dict(), SECRET_KEY, ALGORITHM)

    return encoded_jwt

async def get_current_user(security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)):
    try:
        # decode token and extract username and expires data
        idxSecret = len(security_scopes.scopes)-1
        payload = jwt.decode(token, SECRET_KEY, algorithms=['HS256'])
        data_token = JwtToken()
        data_token.name = payload.get("name")
        data_token.username = payload.get("username")
        data_token.email = payload.get("email")
        data_token.exp = payload.get("exp")
        data_token.userId = payload.get("userId")
        data_token.schoolId = payload.get("schoolId")
        data_token.bankCentralId = payload.get("bankCentralId")
        data_token.bankAreaId = payload.get("bankAreaId")
        data_token.bankBranchId = payload.get("bankBranchId")
        # data_token.teamAreaId = payload.get("teamAreaId")

        data_token.role = payload.get("role")
        if str(security_scopes.scopes[0]).lower() == "*":
            print("Semua Role memiliki akases")
        elif str(data_token.role).upper() in str(security_scopes.scopes):
            print(f"Role {str(data_token.role).lower()} memiliki akases")
        elif str(data_token.role).upper() not in str(security_scopes.scopes):
            raise ROLE_EXCPETION
    except jwt.PyJWTError:
        raise CREDENTIALS_EXCEPTION
    return data_token

async def create_refresh_token(response, token, ALGORITHM='HS256'):
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        data_token = JwtToken()
        data_token.name = payload.get("name")
        data_token.username = payload.get("username")
        data_token.email = payload.get("email")
        data_token.exp = payload.get("exp")
        data_token.userId = payload.get("userId")
        data_token.schoolId = payload.get("schoolId")
        data_token.bankCentralId = payload.get("bankCentralId")
        data_token.bankAreaId = payload.get("bankAreaId")
        data_token.bankBranchId = payload.get("bankBranchId")
        # data_token.teamAreaId = payload.get("teamAreaId")
        data_token.role = payload.get("role")
        # cek token sekarang masih on atau gak
        if datetime.utcfromtimestamp(payload.get("exp")) > datetime.utcnow():
            # cek username masih ada atau tidak
            # if(await GetUserOr404ByUsername(payload.get("username"))):
            access_token = await create_access_token(data_token, 15, SECRET_KEY)
            # print(access_token)
            btoken = "Bearer " + str(access_token)
            response.headers["Authorization"] = btoken
            return {"access_token": access_token}

    except Exception:
        raise CREDENTIALS_EXCEPTION
    raise CREDENTIALS_EXCEPTION

async def get_config():
    return {
        'env' : ENVIRONMENT,
        'alg' : ALGORITHM,
        'url' : BASEURL
    }



import yaml
import consul
ENVIRONMENT='devel'
#Nilai ENVIRONMENT akan berubah otomatis pada waktu build di AWS-EKS
def load_config() -> dict:
    if ENVIRONMENT=='production' or ENVIRONMENT=='staging':
        #jangan ubah, proses build prod akan otomatis replace string host='consul-staging' menjadi 'consul' (prod)
        #link setting consul staging: http://10.8.0.1:8501/ui/dc1/kv/config/crm/setting/edit
        c = consul.Consul(host='consul-staging', port=8500)
        index, data = c.kv.get('config/crm/setting')
        config = yaml.load(data['Value'],Loader=yaml.SafeLoader)
        return config
    else:
        #setting config memakai file local
        with open('config/setting.yml') as yaml_file:
            conf = yaml.load(yaml_file.read(), Loader=yaml.SafeLoader)
        return conf


from setuptools import find_packages, setup

setup(
    name='crmv2_auth_lib',
    packages=find_packages(include=['crmv2_auth_lib']),
    version='0.2.3',
    description='Auth Library CRMV2 Staging',
    author='mrizkyff',
    license='MIT',
    install_requires=[
        'pyjwt',
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='test'
)


from motor.motor_asyncio import AsyncIOMotorClient
import redis
# from .setting import load_config
from setting import load_config

CONF = load_config()

DEBUG = CONF.get("fastapi", dict())["debug"]
SECRET_KEY = CONF.get("fastapi", dict())["key"]
ENVIRONMENT = CONF.get("fastapi", dict())["environment"]
PROJECT_NAME = CONF.get("fastapi", dict())["projectName"]
ALGORITHM = CONF.get("fastapi", dict())["algorithm"]
BASEURL = CONF.get("fastapi", dict())["baseUrl"]

MGDB_CLIENT = AsyncIOMotorClient(
    host=CONF.get("mongodb", dict())["HOST"],
    port=CONF.get("mongodb", dict())["PORT"],
    username=CONF.get("mongodb", dict())["USER"],
    password=CONF.get("mongodb", dict())["PASSWORD"],
)

MGDB = MGDB_CLIENT[CONF.get("mongodb", dict())["NAME"]]

def close_db_client():
    MGDB_CLIENT.close()

REDQUE = redis.Redis(
    host=CONF.get("redis-queue", dict())["HOST"],
    port=CONF.get("redis-queue", dict())["PORT"],
    db=0,
)

REDATA = redis.Redis(
    host=CONF.get("redis-data", dict())["HOST"],
    port=CONF.get("redis-data", dict())["PORT"],
    db=0,
)

# HOST_TAP_DEVICE = "https://api.dev.katalis.info"

# mqtt_config = MQTTConfig(
#     host=CONF.get("mqtt-chat", dict())["HOST"],
#     port=CONF.get("mqtt-chat", dict())["PORT"],
#     keepalive=CONF.get("mqtt-chat", dict())["KEEPALIVE"],
#     username=CONF.get("mqtt-chat", dict())["USERNAME"],
#     password=CONF.get("mqtt-chat", dict())["PASSWORD"],
# )

# CONMQTT = FastMQTT(config=mqtt_config)
